@@ -315,7 +315,8 @@ static int64_t stream_duration(AVStream *stream, AVFormatContext *format)
 }
 
 
-media_object::media_object() : _ffmpeg(NULL)
+media_object::media_object(bool always_convert_to_bgra32) :
+    _always_convert_to_bgra32(always_convert_to_bgra32), _ffmpeg(NULL)
 {
     avdevice_register_all();
     av_register_all();
@@ -382,9 +383,10 @@ void media_object::set_video_frame_template(int index)
     video_frame_template.color_space = video_frame::srgb;
     video_frame_template.value_range = video_frame::u8_full;
     video_frame_template.chroma_location = video_frame::center;
-    if (video_codec_ctx->pix_fmt == PIX_FMT_YUV444P
-            || video_codec_ctx->pix_fmt == PIX_FMT_YUV422P
-            || video_codec_ctx->pix_fmt == PIX_FMT_YUV420P)
+    if (!_always_convert_to_bgra32
+            && (video_codec_ctx->pix_fmt == PIX_FMT_YUV444P
+                || video_codec_ctx->pix_fmt == PIX_FMT_YUV422P
+                || video_codec_ctx->pix_fmt == PIX_FMT_YUV420P))
     {
         if (video_codec_ctx->pix_fmt == PIX_FMT_YUV444P)
         {
@@ -418,9 +420,10 @@ void media_object::set_video_frame_template(int index)
             video_frame_template.chroma_location = video_frame::topleft;
         }
     }
-    else if (video_codec_ctx->pix_fmt == PIX_FMT_YUVJ444P
-            || video_codec_ctx->pix_fmt == PIX_FMT_YUVJ422P
-            || video_codec_ctx->pix_fmt == PIX_FMT_YUVJ420P)
+    else if (!_always_convert_to_bgra32
+            && (video_codec_ctx->pix_fmt == PIX_FMT_YUVJ444P
+                || video_codec_ctx->pix_fmt == PIX_FMT_YUVJ422P
+                || video_codec_ctx->pix_fmt == PIX_FMT_YUVJ420P))
     {
         if (video_codec_ctx->pix_fmt == PIX_FMT_YUVJ444P)
         {
@@ -574,7 +577,7 @@ void media_object::set_audio_blob_template(int index)
     AVCodecContext *audio_codec_ctx = _ffmpeg->audio_codec_ctxs[index];
     audio_blob &audio_blob_template = _ffmpeg->audio_blob_templates[index];
 
-    AVMetadataTag *tag = av_metadata_get(audio_stream->metadata, "language", NULL, AV_METADATA_IGNORE_SUFFIX);
+    AVDictionaryEntry *tag = av_dict_get(audio_stream->metadata, "language", NULL, AV_DICT_IGNORE_SUFFIX);
     if (tag)
     {
         audio_blob_template.language = tag->value;
@@ -624,7 +627,7 @@ void media_object::set_subtitle_box_template(int index)
     //AVCodecContext *subtitle_codec_ctx = _ffmpeg->subtitle_codec_ctxs[index];
     subtitle_box &subtitle_box_template = _ffmpeg->subtitle_box_templates[index];
 
-    AVMetadataTag *tag = av_metadata_get(subtitle_stream->metadata, "language", NULL, AV_METADATA_IGNORE_SUFFIX);
+    AVDictionaryEntry *tag = av_dict_get(subtitle_stream->metadata, "language", NULL, AV_DICT_IGNORE_SUFFIX);
     if (tag)
     {
         subtitle_box_template.language = tag->value;
@@ -643,16 +646,11 @@ void media_object::open(const std::string &url, const device_request &dev_reques
 
     /* Set format and parameters for device input */
     AVInputFormat *iformat = NULL;
-    AVFormatParameters iparams;
-    std::memset(&iparams, 0, sizeof(iparams));
-    iparams.pix_fmt = PIX_FMT_NONE;
-    bool use_iparams = false;
+    AVDictionary *iparams = NULL;
     switch (dev_request.device)
     {
     case device_request::firewire:
         iformat = av_find_input_format("libdc1394");
-        // libdc1394 requires pix_fmt = PIX_FMT_NONE to choose a suitable default pixel format.
-        use_iparams = true;
         break;
     case device_request::x11:
         iformat = av_find_input_format("x11grab");
@@ -660,22 +658,10 @@ void media_object::open(const std::string &url, const device_request &dev_reques
     case device_request::sys_default:
 #if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
         iformat = av_find_input_format("vfwcap");
-        // vfwcap requires a time_base parameter. Set the default to 1/25.
-        iparams.time_base.num = 1;
-        iparams.time_base.den = 25;
-        use_iparams = true;
 #elif defined __FreeBSD__ || defined __NetBSD__ || defined __OpenBSD__ || defined __APPLE__
         iformat = av_find_input_format("bktr");
-        // bktr requires width, height, time_base parameters.
-        // Set the defaults to 640x480 and 1/25.
-        iparams.width = 640;
-        iparams.height = 480;
-        iparams.time_base.num = 1;
-        iparams.time_base.den = 25;
-        use_iparams = true;
 #else
         iformat = av_find_input_format("video4linux2");
-        // video4linux2 can use default parameters.
 #endif
         break;
     case device_request::no_device:
@@ -688,30 +674,26 @@ void media_object::open(const std::string &url, const device_request &dev_reques
                 : dev_request.device == device_request::x11 ? _("X11")
                 : _("default")));
     }
-    if (_is_device
-            && ((dev_request.width != 0 && dev_request.height != 0)
-                || (dev_request.frame_rate_num != 0 && dev_request.frame_rate_den != 0)))
+    if (_is_device && dev_request.width != 0 && dev_request.height != 0)
     {
-        use_iparams = true;
-        if (dev_request.width != 0 && dev_request.height != 0)
-        {
-            iparams.width = dev_request.width;
-            iparams.height = dev_request.height;
-        }
-        if (dev_request.frame_rate_num != 0 && dev_request.frame_rate_den != 0)
-        {
-            iparams.time_base.den = dev_request.frame_rate_num;
-            iparams.time_base.num = dev_request.frame_rate_den;
-        }
+        av_dict_set(&iparams, "video_size", str::asprintf("%dx%d",
+                    dev_request.width, dev_request.height).c_str(), 0);
+    }
+    if (_is_device && dev_request.frame_rate_num != 0 && dev_request.frame_rate_den != 0)
+    {
+        av_dict_set(&iparams, "framerate", str::asprintf("%d/%d",
+                    dev_request.frame_rate_num, dev_request.frame_rate_den).c_str(), 0);
     }
 
     /* Open the input */
-    if ((e = av_open_input_file(&_ffmpeg->format_ctx, _url.c_str(), iformat, 0,
-                    use_iparams ? &iparams : NULL)) != 0)
+    _ffmpeg->format_ctx = NULL;
+    if ((e = avformat_open_input(&_ffmpeg->format_ctx, _url.c_str(), iformat, &iparams)) != 0)
     {
+        av_dict_free(&iparams);
         throw exc(str::asprintf(_("%s: %s"),
                     _url.c_str(), my_av_strerror(e).c_str()));
     }
+    av_dict_free(&iparams);
     if (_is_device)
     {
         // For a camera device, do not read ahead multiple packets, to avoid a startup delay.
@@ -725,8 +707,8 @@ void media_object::open(const std::string &url, const device_request &dev_reques
     av_dump_format(_ffmpeg->format_ctx, 0, _url.c_str(), 0);
 
     /* Metadata */
-    AVMetadataTag *tag = NULL;
-    while ((tag = av_metadata_get(_ffmpeg->format_ctx->metadata, "", tag, AV_METADATA_IGNORE_SUFFIX)))
+    AVDictionaryEntry *tag = NULL;
+    while ((tag = av_dict_get(_ffmpeg->format_ctx->metadata, "", tag, AV_DICT_IGNORE_SUFFIX)))
     {
         _tag_names.push_back(tag->key);
         _tag_values.push_back(tag->value);
@@ -739,30 +721,32 @@ void media_object::open(const std::string &url, const device_request &dev_reques
             && i < static_cast<unsigned int>(std::numeric_limits<int>::max()); i++)
     {
         _ffmpeg->format_ctx->streams[i]->discard = AVDISCARD_ALL;        // ignore by default; user must activate streams
-        if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
+        AVCodecContext *codec_ctx = _ffmpeg->format_ctx->streams[i]->codec;
+        AVCodec *codec = NULL;
+        if (_ffmpeg->format_ctx->streams[i]->codec->codec_id != CODEC_ID_TEXT
+                && (!(codec = avcodec_find_decoder(_ffmpeg->format_ctx->streams[i]->codec->codec_id))
+                    || (e = avcodec_open(codec_ctx, codec)) < 0))
+        {
+            msg::wrn(_("%s stream %d: Cannot open %s: %s"), _url.c_str(), i,
+                    _ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO ? _("video codec")
+                    : _ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO ? _("audio codec")
+                    : _ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_SUBTITLE ? _("subtitle codec")
+                    : _("data"),
+                    codec ? my_av_strerror(e).c_str() : _("codec not supported"));
+        }
+        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
         {
             _ffmpeg->video_streams.push_back(i);
             int j = _ffmpeg->video_streams.size() - 1;
             msg::dbg(_url + " stream " + str::from(i) + " is video stream " + str::from(j) + ".");
-            _ffmpeg->video_codec_ctxs.push_back(_ffmpeg->format_ctx->streams[i]->codec);
+            _ffmpeg->video_codec_ctxs.push_back(codec_ctx);
             if (_ffmpeg->video_codec_ctxs[j]->width < 1 || _ffmpeg->video_codec_ctxs[j]->height < 1)
             {
                 throw exc(str::asprintf(_("%s video stream %d: Invalid frame size."),
                             _url.c_str(), j + 1));
             }
-            _ffmpeg->video_codecs.push_back(avcodec_find_decoder(_ffmpeg->video_codec_ctxs[j]->codec_id));
-            if (!_ffmpeg->video_codecs[j])
-            {
-                throw exc(str::asprintf(_("%s video stream %d: Unsupported video codec."),
-                            _url.c_str(), j + 1));
-            }
+            _ffmpeg->video_codecs.push_back(codec);
             _ffmpeg->video_codec_ctxs[j]->thread_count = video_decoding_threads();
-            if ((e = avcodec_open(_ffmpeg->video_codec_ctxs[j], _ffmpeg->video_codecs[j])) < 0)
-            {
-                _ffmpeg->video_codecs[j] = NULL;
-                throw exc(str::asprintf(_("%s video stream %d: Cannot open video codec: %s"),
-                            _url.c_str(), j + 1, my_av_strerror(e).c_str()));
-            }
             // Determine frame template.
             _ffmpeg->video_frame_templates.push_back(video_frame());
             set_video_frame_template(j);
@@ -812,19 +796,8 @@ void media_object::open(const std::string &url, const device_request &dev_reques
             _ffmpeg->audio_streams.push_back(i);
             int j = _ffmpeg->audio_streams.size() - 1;
             msg::dbg(_url + " stream " + str::from(i) + " is audio stream " + str::from(j) + ".");
-            _ffmpeg->audio_codec_ctxs.push_back(_ffmpeg->format_ctx->streams[i]->codec);
-            _ffmpeg->audio_codecs.push_back(avcodec_find_decoder(_ffmpeg->audio_codec_ctxs[j]->codec_id));
-            if (!_ffmpeg->audio_codecs[j])
-            {
-                throw exc(str::asprintf(_("%s audio stream %d: Unsupported audio codec."),
-                            _url.c_str(), j + 1));
-            }
-            if ((e = avcodec_open(_ffmpeg->audio_codec_ctxs[j], _ffmpeg->audio_codecs[j])) < 0)
-            {
-                _ffmpeg->audio_codecs[j] = NULL;
-                throw exc(str::asprintf(_("%s audio stream %d: Cannot open audio codec: %s"),
-                            _url.c_str(), j + 1, my_av_strerror(e).c_str()));
-            }
+            _ffmpeg->audio_codec_ctxs.push_back(codec_ctx);
+            _ffmpeg->audio_codecs.push_back(codec);
             _ffmpeg->audio_blob_templates.push_back(audio_blob());
             set_audio_blob_template(j);
             _ffmpeg->audio_decode_threads.push_back(audio_decode_thread(_url, _ffmpeg, j));
@@ -844,27 +817,10 @@ void media_object::open(const std::string &url, const device_request &dev_reques
             _ffmpeg->subtitle_streams.push_back(i);
             int j = _ffmpeg->subtitle_streams.size() - 1;
             msg::dbg(_url + " stream " + str::from(i) + " is subtitle stream " + str::from(j) + ".");
-            _ffmpeg->subtitle_codec_ctxs.push_back(_ffmpeg->format_ctx->streams[i]->codec);
-            if (_ffmpeg->subtitle_codec_ctxs[j]->codec_id == CODEC_ID_TEXT)
-            {
-                // CODEC_ID_TEXT does not have any decoder; it is just UTF-8 text in the packet data.
-                _ffmpeg->subtitle_codecs.push_back(NULL);
-            }
-            else
-            {
-                _ffmpeg->subtitle_codecs.push_back(avcodec_find_decoder(_ffmpeg->subtitle_codec_ctxs[j]->codec_id));
-                if (!_ffmpeg->subtitle_codecs[j])
-                {
-                    throw exc(str::asprintf(_("%s subtitle stream %d: Unsupported subtitle codec."),
-                                _url.c_str(), j + 1));
-                }
-                if ((e = avcodec_open(_ffmpeg->subtitle_codec_ctxs[j], _ffmpeg->subtitle_codecs[j])) < 0)
-                {
-                    _ffmpeg->subtitle_codecs[j] = NULL;
-                    throw exc(str::asprintf(_("%s subtitle stream %d: Cannot open subtitle codec: %s"),
-                                _url.c_str(), j + 1, my_av_strerror(e).c_str()));
-                }
-            }
+            _ffmpeg->subtitle_codec_ctxs.push_back(codec_ctx);
+            // CODEC_ID_TEXT does not have any decoder; it is just UTF-8 text in the packet data.
+            _ffmpeg->subtitle_codecs.push_back(
+                    _ffmpeg->subtitle_codec_ctxs[j]->codec_id == CODEC_ID_TEXT ? NULL : codec);
             _ffmpeg->subtitle_box_templates.push_back(subtitle_box());
             set_subtitle_box_template(j);
             _ffmpeg->subtitle_decode_threads.push_back(subtitle_decode_thread(_url, _ffmpeg, j));
