@@ -189,6 +189,7 @@ public:
     int height() { return _channel->getPixelViewport().h; }
     int pos_x() { return 0; }
     int pos_y() { return 0; }
+    bool fullscreen() { return false; }
     void center() { }
     void enter_fullscreen(int) { }
     void exit_fullscreen() { }
@@ -201,7 +202,7 @@ public:
     video_output_eq_channel(eq::Channel *channel,
             float canvas_width, float canvas_height,
             float canvas_video_area_w, float canvas_video_area_h) :
-        video_output(false),
+        video_output(),
         _channel(channel),
         _canvas_width(canvas_width),
         _canvas_height(canvas_height),
@@ -415,7 +416,7 @@ public:
         _eq_init_data(),
         _eq_frame_data(),
         _player(),
-        _controller(false),
+        _controller(),
         frame_template()
     {
     }
